/*
 * A program that uses Monte Carlo simulation to estimate the number
 * of of three letter words that appear in the online dictionary.
 * Uses "isword" provided for this CS50 assignment.
 *
 * Penny Ellard
 * 9/19/97
 *
 */

#include	<stdio.h>
#include	"genlib.h"
#include	"simpio.h"
#include	"time.h"

/* If the student chooses to use the Roberts random library,
 * make sure they do this, instead of including time.h
 *
 * #include	"random.h"
 */

/* Define 
 *	the number of characters in a word (NUM_CHARS = 3)
 *	the number of characters we pick from (MAX_CHAR = 26)
 *	the number of possible combinations (NUM_WORDS)
 *	the 'lo' value passed to rand_int (FIRST_LETTER = 'a')
 *	the 'hi' value passed to rand_int (LAST_LETTER = 'z')
 */

#define	NUM_CHARS	3
#define	MAX_CHAR	26
#define NUM_WORDS	(MAX_CHAR * MAX_CHAR * MAX_CHAR)
#define FIRST_LETTER	'a'
#define LAST_LETTER	'z'

/* Function declarations, all found within monte.c */

int get_num_samples (void);
int run_simulation (int);
void print_solution (int, int);
void generate_word (char word [], int size);
int is_word (char word[]);
int rand_int (int low, int high);


/*
 * main - gets the number of samples to run, runs the simulation,
 *		then prints the solution
 */

void main () {

	int num_samples, num_found;

	num_samples = get_num_samples ();

	printf ("Samples\tSuccesses\tEstimate\n");
	num_found = run_simulation (num_samples);

	printf ("================================\n");
	print_solution (num_found, num_samples);
}

/* 
 * get_num_samples - prompts the user for the number of samples,
 *		then reads the input, checks that it is greater
 *		than 0, then returns the value.
 */

int get_num_samples () {

	int num_samples;
	bool valid_entry;

	valid_entry = FALSE;
	while (!valid_entry) {

		printf ("How many samples should I try? ");
		num_samples = GetInteger ();

		if (num_samples < 1) {
			printf ("Please enter a number greater than 0!\n");
		} else {
			valid_entry = TRUE;
		}
	}

	return (num_samples);
}

/*
 * run_simulation - seeds the random number generator, calls
 *		generate_word to generate a random word, checks to
 *		see if that word is in the dictionary, then prints
 *		out the status if the sample number is a power of 2.
 *		Returns the number of samples found in the dictionary.
 */

int run_simulation (int num_samples) {

	int i;
	int num_found = 0;
	char rand_word[NUM_CHARS + 1];		/* one for '\0' */
	int next_power = 1;			/* powers of 2 */

	srand ((unsigned) ((time_t *) 0));

	/* if they use Roberts, replace the previous line with
	 *
	 * Randomize ();
	 */

	for (i = 1; i <= num_samples; i++) {
		generate_word (rand_word, NUM_CHARS + 1);
		if (is_word (rand_word)) {
			num_found++;
		}
		if ((i == next_power) && (i != num_samples)) {
			print_solution (num_found, i);

			next_power = next_power * 2;
		}
	}

	return (num_found);
}

/* 
 * generate_word - takes as an argument a char array that it will fill
 * 		in with random, lower-case letters, and the size of
 *		the array.
 *		Called by run_simulation
 */

void generate_word (char word [], int size) {

	int i;

	for (i = 0; i < size - 1; i++) {

		/* If they use Roberts, use this instead :
		 *
		 * rand_char_i = RandomInteger (FIRST_LETTER, LAST_LETTER);
		 */

		word[i] = (char) rand_int (FIRST_LETTER, LAST_LETTER);
	}
	word [i] = '\0';
}

/*
 * rand_int
 * A random number generator taken from randtest.c, shown during
 * lecture.
 * Follows the basic algorithm of Normalize, Scale, Truncate and Translate.
 *      Input: high and low values.
 *      Output: random integer.
 */

int rand_int(int low, int high)
{
        int range, val;

        range = high - low + 1;
        val = low + (int)(range * (double)rand() / ((double)(RAND_MAX) + 1));
        return(val);
}


/* 
 * print_solution - prints out the number of samples run, the
 * 		number of samples found, and the estimate of
 *		the number of words in the dictionary
 */

void print_solution (int num_found, int num_samples) {

	int estimate = (int) ((double)num_found / num_samples * NUM_WORDS);

	printf ("%d\t%d\t\t%d\n", num_samples, num_found, estimate);

}
